#Load the dataset
xtrain<-read.table("14cancer.xtrain.xtrain",sep="", header=F)
ytrain<-read.table("14cancer.ytrain.ytrain",sep="", header=F) # the order is correct
xs=scale(xtrain) # scale by patient

#For illustrative purposes, reduce the number of features to 2684 with largest sd
featuressd=apply(xs,1,sd)
selected=(1:nrow(xs))[featuressd>0.45] #0.45 gives 2684 features

#Form new datasets with only selected features
xtrain=t(xs[selected,])

# Construct matrix W
p=ncol(xtrain)
G=max(ytrain)
W=matrix(0,p,p)
for (g in 1:G){
    W=W+(sum(ytrain==g)-1)*cov(xtrain[ytrain==g,])
}
# Scale so that diag(W)=I
xtrains=scale(xtrain,center=F,scale=sqrt(diag(W)))
# Construct matrix B
B=matrix(0,p,p)
for (g in 1:G){
    B=B+sum(ytrain==g)*(colMeans(xtrains[ytrain==g,])-colMeans(xtrains))%*%t(colMeans(xtrains[ytrain==g,])-colMeans(xtrains))
}
B=B/length(ytrain)

# Apply Algorithm~1 for the range of tuning parameters
source("Algorithm1.R")
lambda_max=max(sqrt(rowSums(B^2)))
nl=80
lambda_intervalC14=seq(0,lambda_max,by=(lambda_max)/(nl-1))
featuresC14=rep(0,nl)
l1_normC14=rep(0,nl)
objectiveC14=rep(0,nl)
#Solve using Algorithm 1 for the range of lambda_values, keep track of the number of selected features
for (i in 1:length(lambda_intervalC14)){
    cat(i)
    lambda=lambda_intervalC14[i]
    v=algorithm1(B,lambda)$v
    featuresC14[i]=sum(v!=0)
    l1_normC14[i]=sum(abs(v))
    objectiveC14[i]=crossprod(v,B%*%v)-lambda*sum(abs(v))
    if (featuresC14[i]==0) break
}

#Make a plot without predicting the drop
pdf(file="14cancerLDA.pdf",width=4,height=4)
library(ggplot2)
scenarioC14=data.frame(lambda=lambda_intervalC14,features=featuresC14)
ggplot(scenarioC14,aes(x=lambda,y=features))+geom_point(size=2)+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("LDA on 14 Cancer Dataset")
dev.off()


# Now try to predict the drop
############
source("CalculateMlambda.R")
# m_lambda
m_lambda=findMlambda(B,lambda_intervalC14) #gives around 14 features for the drop
pdf(file="14CancerLDAmlambda.pdf",width=4,height=4)
library(ggplot2)
scenarioC14=data.frame(lambda=lambda_intervalC14,features=featuresC14,Mlambda=m_lambda)
ggplot(scenarioC14,aes(x=lambda,y=features))+geom_point(size=2)+geom_line(data=scenarioC14,aes(x=lambda,y=Mlambda),linetype="dashed",size=2,colour="blue")+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("LDA on 14 Cancer Dataset")
dev.off()

# Constrained version 
##############
Q=B
library(irlba)
tmp=irlba(Q,nu=1,nv=0)
l=tmp$u
gamma=tmp$d
tau_interval14=seq(1,sum(abs(l)),by=(sum(abs(l)))/50)
features14tau=rep(0,length(tau_interval14))
source("Algorithm1.R")

#Solve using Algorithm 2 for the range of tau_values with binary search in C
features14tau_binc=rep(0,length(features14tau))
for (i in 1:length(tau_interval14)){
    cat(i)
    tau=tau_interval14[i]
    v=algorithm2_binc(Q,tau)$v
    features14tau_binc[i]=sum(v!=0)
}

#Make a plot of number of selected variables versus tuning parameter for Scenario 2
library(ggplot2)
pdf(file="14CancerLDAconstrained.pdf",width=4,height=4)
scenario14tau=data.frame(lambda=-tau_interval14,features=features14tau)
ggplot(scenario14tau,aes(x=lambda,y=features))+geom_point(size=2)+ylab("Number of selected variables")+xlab(expression(-tau))+ggtitle("LDA on 14 Cancer Dataset")
dev.off()
