#Load the dataset
xtrain<-read.table("14cancer.xtrain.xtrain",sep="", header=F)
ytrain<-read.table("14cancer.ytrain.ytrain",sep="", header=F) # the order is correct
xs=scale(xtrain) # scale by patient

#For illustrative purposes, reduce the number of features to 2684 with largest sd
featuressd=apply(xs,1,sd)
selected=(1:nrow(xs))[featuressd>0.45] #0.45 gives 2684 features

#Form new datasets with only selected features
xtrain=t(xs[selected,])

#Standardize the data
xtrains=scale(xtrain)
Q=crossprod(xtrains)/(nrow(xtrains)-1)

# Apply Algorithm~1 for the range of tuning parameters
source("Algorithm1.R")
lambda_max=max(sqrt(rowSums(Q^2)))
nl=80
lambda_intervalC14PCA=seq(0,lambda_max,by=(lambda_max)/(nl-1))
featuresC14PCA=rep(0,nl)
l1_normC14PCA=rep(0,nl)
objectiveC14PCA=rep(0,nl)
#Solve using Algorithm 1 for the range of lambda_values, keep track of the number of selected features
for (i in 1:length(lambda_intervalC14PCA)){
    cat(i)
    lambda=lambda_intervalC14PCA[i]
    v=algorithm1(Q,lambda)$v
    featuresC14PCA[i]=sum(v!=0)
    l1_normC14PCA[i]=sum(abs(v))
    objectiveC14PCA[i]=crossprod(v,Q%*%v)-lambda*sum(abs(v))
    if (featuresC14PCA[i]==0) break
}

pdf(file="14cancerPCA.pdf",width=4,height=4)
library(ggplot2)
scenarioC14PCA=data.frame(lambda=lambda_intervalC14PCA,features=featuresC14PCA)
ggplot(scenarioC14PCA,aes(x=lambda,y=features))+geom_point(size=2)+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("PCA on 14 Cancer Dataset")
dev.off()

# Now try to predict the drop
############
source("CalculateMlambda.R")
# m_lambda
m_lambda=findMlambda(Q,lambda_intervalC14PCA) #gives around 267 for the drop
pdf(file="14CancerPCAmlambda.pdf",width=4,height=4)
library(ggplot2)
scenarioC14PCA=data.frame(lambda=lambda_intervalC14PCA,features=featuresC14PCA,Mlambda=m_lambda)
ggplot(scenarioC14PCA,aes(x=lambda,y=features))+geom_point(size=2)+geom_line(data=scenarioC14PCA,aes(x=lambda,y=Mlambda),linetype="dashed",size=2,colour="blue")+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("PCA on 14 Cancer Dataset")
dev.off()

#Find eigenvector and eigenvalue of Q
library(irlba)
tmp=irlba(Q,nu=1,nv=0)
l=tmp$u
gamma=tmp$d
tau_interval14PCA=seq(1,sum(abs(l)),by=(sum(abs(l)))/50)
features14PCAtau=rep(0,length(tau_interval14PCA))
source("Algorithm1.R")

#Solve using Algorithm 2 for the range of tau_values, keep track of the number of selected features
for (i in 1:length(tau_interval14PCA)){
    cat(i)
    tau=tau_interval14PCA[i]
    v=algorithm2_binc(Q,tau)$v
    features14PCAtau[i]=sum(v!=0)
}

#Make a plot of number of selected variables versus tuning parameter for Scenario 2
library(ggplot2)
pdf(file="14CancerPCAconstrained.pdf",width=4,height=4)
scenario14PCAtau=data.frame(lambda=-tau_interval14PCA,features=features14PCAtau)
ggplot(scenario14PCAtau,aes(x=lambda,y=features))+geom_point(size=2)+ylab("Number of selected variables")+xlab(expression(-tau))+ggtitle("PCA on 14 Cancer Dataset")
dev.off()

