#Make sure all dynamic libraries for C code are loaded before running

# Code that uses Algorithm1 to solve ell_1 penalized problem
algorithm1<-function(Q,lambda,v_init=NULL,max_iter=1000, eps=1e-4){
    library(irlba)
    p=nrow(Q)
    if (is.null(v_init)){
        # initialize v as the leading eigenvector of Q
        v=irlba(Q,nu=1,nv=0)$u
        # normalize v to have L2 norm equal to one
        v=v/(sqrt(as.numeric(crossprod(v))))
    } else v=v_init
    # current value of objective function
    fobj=t(v)%*%Q%*%v-lambda*sum(abs(v))
    iter=0
    error=100
    while ((error>eps)&(iter<max_iter)){
        iter=iter+1
        tmp=Q%*%v
        # new update
        v=sign(tmp)*pmax(abs(tmp)-lambda/2,0)
        # check that it's not zero before standardizing
        if (as.numeric(crossprod(v))!=0){
            v=v/sqrt(as.numeric(crossprod(v)))
        }
        # new value of objective
        fnew=crossprod(v,Q%*%v)-lambda*sum(abs(v))
        # change in objective
        error=fnew-fobj
        fobj=fnew
    }
    if (iter>=max_iter) warning("Failed to converge, try increasing the number of iterations max_iter")
    # if the value of objective is below zero, return 0 as a better solution
    if (fobj<0){
        return(list(v=rep(0,p),fobj=0))
    }else{
        return(list(v=v,fobj=fobj))
    }
}
# Alternative C code
source("Algorithm1_c.R")

# Code that uses Algorithm1 to solve ell_1 constrained problem

#binary search for each iteration k
binsearch<-function(tmp,lmax,tau,eps=1e-2,max_iter=5000){
    lmin=0
    lambda=(lmax+lmin)/2
    v=sign(tmp)*pmax(abs(tmp)-lambda/2,0)
    # check that it's not zero before standardizing
    if (sum(v^2)!=0){
        v=v/sqrt(sum(v^2))
    }
    # current l1norm
    l1norm=sum(abs(v))
    iter=0
    while (((abs(l1norm-tau))>eps)&(iter<max_iter)){
        iter=iter+1
        # the current solution has larger norm then it should, need to threshold more
        if (l1norm>tau){
            lmin=lambda
            lambda=(lmax+lmin)/2
            v=sign(tmp)*pmax(abs(tmp)-lambda/2,0)
            # check that it's not zero before standardizing
            if (sum(v^2)!=0){
                v=v/sqrt(sum(v^2))
            }
            l1norm=sum(abs(v))
        # the current solution has lower norm then it should, need to threshold less
        }else{
            lmax=lambda
            lambda=(lmax+lmin)/2
            v=sign(tmp)*pmax(abs(tmp)-lambda/2,0)
            # check that it's not zero before standardizing
            if (sum(v^2)!=0){
                v=v/sqrt(sum(v^2))
            }
            l1norm=sum(abs(v))
        }
    }
    if (iter>=max_iter) warning("Failed to find appropriate tuning parameter")
    return(lambda)
}

#modified algorithm 1
algorithm2<-function(Q,tau,max_iter=1000, eps=1e-4){
    library(irlba)
    p=nrow(Q)
    # initialize v as the leading eigenvector of Q
    v=irlba(Q,nu=1,nv=0)$u
    # normalize v to have L2 norm equal to one
    v=v/(sqrt(sum(v^2)))
    # current value of objective function
    fobj=0
    iter=0
    error=100
    while ((error>eps)&(iter<max_iter)){
        iter=iter+1
        tmp=Q%*%v
        lambda=binsearch(tmp,lmax=2*max(abs(tmp)),tau)
        # new update
        v=sign(tmp)*pmax(abs(tmp)-lambda/2,0)
        # check that it's not zero before standardizing
        if (sum(v^2)!=0){
            v=v/sqrt(sum(v^2))
        }
        # new value of objective
        fnew=t(v)%*%Q%*%v
        # change in objective
        error=fnew-fobj
        fobj=fnew
    }
    if (iter>=max_iter) warning("Failed to converge, try increasing the number of iterations max_iter")
    # if the value of objective is below zero, return 0 as better solution
    if (fobj<0){
        return(rep(0,p))
    }else{
        return(v)
    }
}

#modified algorighm 1 with binary search in C
algorithm2_binc<-function(Q,tau,max_iter=1000, eps=1e-4){
    library(irlba)
    source("BinarySearch_c.R")
    p=nrow(Q)
    # initialize v as the leading eigenvector of Q
    v=irlba(Q,nu=1,nv=0)$u
    # normalize v to have L2 norm equal to one
    v=v/(sqrt(sum(v^2)))
    # current value of objective function
    fobj=0
    iter=0
    error=100
    while ((error>eps)&(iter<max_iter)){
        iter=iter+1
        tmp=Q%*%v
        lambda=binsearch_c(tmp,lmax=2*max(abs(tmp)),tau)
        # new update
        v=sign(tmp)*pmax(abs(tmp)-lambda/2,0)
        # check that it's not zero before standardizing
        if (sum(v^2)!=0){
            v=v/sqrt(sum(v^2))
        }
        # new value of objective
        fnew=t(v)%*%Q%*%v
        # change in objective
        error=fnew-fobj
        fobj=fnew
    }
    if (iter>=max_iter) warning("Failed to converge, try increasing the number of iterations max_iter")
    # if the value of objective is below zero, return 0 as better solution
    if (fobj<0){
        return(rep(0,p))
    }else{
        return(v)
    }
}