# Empirical simulations for Section~3 with synthetic data
# Consider p=500 and p=2000
# C=I for all simulations
# rank(Q)=1 and 50
source("Algorithm1.R")
source("CalculateMlambda.R")
library(ggplot2)
library(mnormt)

# Rank(Q)=1
# p=500, C=I, l_i come from uniform distribution on [0,1] and then normalized as l^tl=1
######################
p=500
set.seed(1)
# draw features of eigenvector l uniformly at random
l=runif(p)
# normalize to have l_2 norm 1
l=as.matrix(l/sqrt(sum(l^2)))
# largest eigenvalue is one
gamma=1
# initialize matrix Q
Q=gamma*(l%*%t(l))

#Apply the current state of theory, lambda_max=max_i ||q_i||_2
lambda_max=max(sqrt(rowSums(Q^2)))
# Interval for tuning parameter lambda
nl=80
lambda_interval1=seq(0,lambda_max,by=lambda_max/(nl-1))
# Vectors that keeps track of the number of non-zero features
features1=rep(0,nl)

#Solve using Algorithm 1 for the range of lambda_values, keep track of the number of selected features
for (i in 1:nl){
    cat(i)
    lambda=lambda_interval1[i]
    v=algorithm1(Q,lambda)$v
    features1[i]=sum(v!=0)
}
# Drop prediction
############
m_lambda=findMlambda(Q,lambda_interval1)

#Make a plot of number of selected variables versus tuning parameter
pdf(file="Scenario1notheory.pdf",height=4,width=4)
scenario1=data.frame(lambda=lambda_interval1,features=features1, Mlambda=m_lambda)
ggplot(scenario1,aes(x=lambda,y=features))+geom_point(size=2)+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("p=500 and rank(Q)=1")
dev.off()

pdf(file="Scenario1mlambda.pdf",height=4,width=4)
scenario1=data.frame(lambda=lambda_interval1,features=features1, Mlambda=m_lambda)
ggplot(scenario1,aes(x=lambda,y=features))+geom_point(size=2)+geom_line(data=scenario1,aes(x=lambda,y=Mlambda),linetype="dashed",size=2,colour="blue")+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("p=500 and rank(Q)=1")
dev.off()


# rank(Q)=1, C=I, p=2000
######################
p=2000
set.seed(300)
# draw features of eigenvector l uniformly at random
l=runif(p)
# normalize to have l_2 norm 1
l=as.matrix(l/sqrt(sum(l^2)))
# largest eigenvalue is one
gamma=1
# initialize matrix Q
Q=gamma*(l%*%t(l))

#Apply the current state of theory, lambda_max=max_i ||q_i||_2
lambda_max=max(sqrt(rowSums(Q^2)))
# Interval for tuning parameter lambda
nl=80
lambda_interval2=seq(0,lambda_max,by=lambda_max/(nl-1))
# Vectors that keeps track of the number of non-zero features
features2=rep(0,nl)

#Solve using Algorithm 1 for the range of lambda_values, keep track of the number of selected features
for (i in 1:nl){
    cat(i)
    lambda=lambda_interval2[i]
    v=algorithm1(Q,lambda)$v
    features2[i]=sum(v!=0)
}
# Drop prediction
############
# m_lambda
m_lambda=findMlambda(Q,lambda_interval2)

#Make a plot of number of selected variables versus tuning parameter 
pdf(file="Scenario2notheory.pdf",height=4,width=4)
scenario2=data.frame(lambda=lambda_interval2,features=features2, Mlambda=m_lambda)
ggplot(scenario2,aes(x=lambda,y=features))+geom_point(size=2)+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("p=2000 and rank(Q)=1")
dev.off()

pdf(file="Scenario2mlambda.pdf",height=4,width=4)
scenario2=data.frame(lambda=lambda_interval2,features=features2, Mlambda=m_lambda)
ggplot(scenario2,aes(x=lambda,y=features))+geom_point(size=2)+geom_line(data=scenario2,aes(x=lambda,y=Mlambda),linetype="dashed",size=2,colour="blue")+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("p=2000 and rank(Q)=1")
dev.off()

# rank(Q)=50, C=I, p=500 
######################
p=500
set.seed(95)
n=50
# Create a covariance matrix 
Xdata=matrix(rnorm(n*p),n,p)
Xdata=scale(Xdata)
Q=cov(Xdata)

#Apply the current state of theory, lambda_max=max_i ||q_i||_2
lambda_max=max(sqrt(rowSums(Q^2)))
# Interval for tuning parameter lambda
nl=80
lambda_interval5=seq(0,lambda_max,by=lambda_max/(nl-1))
# Vectors that keeps track of the number of non-zero features
features5=rep(0,nl)

#Solve using Algorithm 1 for the range of lambda_values, keep track of the number of selected features
for (i in 1:nl){
    cat(i)
    lambda=lambda_interval5[i]
    v=algorithm1(Q,lambda)$v
    features5[i]=sum(v!=0)
    if (features5[i]==0) break
}
# Drop prediction
############
# m_lambda
m_lambda=findMlambda(Q,lambda_interval5)

#Make a plot of number of selected variables versus tuning parameter 
pdf(file="../figs/Scenario5notheory.pdf",width=4,height=4)
scenario5=data.frame(lambda=lambda_interval5,features=features5, Mlambda=m_lambda)
ggplot(scenario5,aes(x=lambda,y=features))+geom_point(size=2)+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("p=500 and rank(Q)=50")
dev.off()

pdf(file="../figs/Scenario5mlambda.pdf",height=4,width=4)
scenario5=data.frame(lambda=lambda_interval5,features=features5, Mlambda=m_lambda)
ggplot(scenario5,aes(x=lambda,y=features))+geom_point(size=2)+geom_line(data=scenario5,aes(x=lambda,y=Mlambda),linetype="dashed",size=2,colour="blue")+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("p=500 and rank(Q)=50")
dev.off()


# rank(Q)=50, C=I, p=2000 
######################
p=2000
set.seed(506)
n=50
# Create a covariance matrix 
Xdata=matrix(rnorm(n*p),n,p)
#svdXdata=svd(Xdata)
Xdata=scale(Xdata)
Q=cov(Xdata)

#Apply the current state of theory, lambda_max=max_i ||q_i||_2
lambda_max=max(sqrt(rowSums(Q^2)))
# Interval for tuning parameter lambda
nl=80
lambda_interval6=seq(0,lambda_max,by=lambda_max/(nl-1))
# Vectors that keeps track of the number of non-zero features
features6=rep(0,nl)

#Solve using Algorithm 1 for the range of lambda_values, keep track of the number of selected features
for (i in 1:nl){
    cat(i)
    lambda=lambda_interval6[i]
    v=algorithm1(Q,lambda)$v
    features6[i]=sum(v!=0)
    if (features6[i]==0) break
}
# Drop prediction
############
# m_lambda
m_lambda=findMlambda(Q,lambda_interval6)

#Make a plot of number of selected variables versus tuning parameter 
pdf(file="../figs/Scenario6notheory.pdf",width=4,height=4)
scenario6=data.frame(lambda=lambda_interval6,features=features6, Mlambda=m_lambda)
ggplot(scenario6,aes(x=lambda,y=features))+geom_point(size=2)+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("p=2000 and rank(Q)=50")
dev.off()

pdf(file="../figs/Scenario6mlambda.pdf",height=4,width=4)
scenario6=data.frame(lambda=lambda_interval6,features=features6, Mlambda=m_lambda)
ggplot(scenario6,aes(x=lambda,y=features))+geom_point(size=2)+geom_line(data=scenario6,aes(x=lambda,y=Mlambda),linetype="dashed",size=2,colour="blue")+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("p=2000 and rank(Q)=50")
dev.off()
