# Simulations that demonstrate empirical drop to zero as a function of tuning parameter lambda
# This time perform r random initializations of v^(0)

## Can use preloaded scenarios so no need to run Algorithm 1 again
# load("ScenariosGlobal.rda")

# rank(Q)=1, C=I, p=500
######################
p=500
set.seed(1)
# draw features of eigenvector l uniformly at random
l=runif(p)
# normalize to have l_2 norm 1
l=as.matrix(l/sqrt(sum(l^2)))
# largest eigenvalue is one
gamma=1
# initialize matrix Q
Q=gamma*(l%*%t(l))

#Apply the current state of theory, lambda_max=max_i ||q_i||_2
lambda_max=max(sqrt(rowSums(Q^2)))
# Interval for tuning parameter lambda
nl=80
lambda_interval=seq(0,lambda_max,by=lambda_max/(nl-1))

# Vectors that keeps track of the number of non-zero features
features=rep(0,length(lambda_interval))
features_global=rep(0,length(lambda_interval))
fobj=rep(0,length(lambda_interval))

# Number of random starts
r=100

# Initialization function for random starts
initializev<-function(p){
    x=rnorm(p)
    return(x/sqrt(crossprod(x)))
}

set.seed(5)
vall0=matrix(0,r,p)
for (i in 1:r){
    vall0[i,]=initializev(p)
}


#Solve using Algorithm 1 for the range of lambda_values, keep track of the number of selected features

source("Algorithm1.R")
featurestmp=matrix(0,length(lambda_interval),r+1)
for (i in 1:length(lambda_interval)){
    cat(i)
    lambda=lambda_interval[i]
    fobjtmp=rep(0,r+1)
    for (j in 1:r){
        v0=vall0[j,]
        out=algorithm1_c(Q,lambda,v_init=v0)
        fobjtmp[j]=out$fobj
        featurestmp[i,j]=sum(out$v!=0)
    }
    out1=algorithm1_c(Q,lambda)
    featurestmp[i,r+1]=sum(out1$v!=0)
    fobjtmp[r+1]=out1$fobj
    features_global[i]=min(featurestmp[i,fobjtmp==max(fobjtmp)])
    fobj[i]=max(fobjtmp)
}

library(ggplot2)
  scenario1=data.frame(lambda=rep(lambda_interval,r+2),features=as.vector(cbind(featurestmp,features_global)),size=c(rep(2,(r+1)*length(lambda_interval)),rep(4,length(lambda_interval))),shape=c(rep(19,(r+1)*length(lambda_interval)),rep(5,length(lambda_interval))),colour=c(rep("red",(r+1)*length(lambda_interval)),rep("black",length(lambda_interval))),alpha=c(rep(0.5,(r+1)*length(lambda_interval)),rep(1,length(lambda_interval))))
pdf(file="../figs/Scenario1globalNew.pdf",width=4,height=4)
ggplot(scenario1,aes(x=lambda,y=features))+geom_point(size=scenario1$size,col=scenario1$colour,shape=scenario1$shape)+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("p=500 and rank(Q)=1")#+theme_bw()
dev.off()



# rank(Q)=1, C=I, p=2000
######################
p=2000
set.seed(300)
# draw features of eigenvector l uniformly at random
l=runif(p)
# normalize to have l_2 norm 1
l=as.matrix(l/sqrt(sum(l^2)))
# largest eigenvalue is one
gamma=1
# initialize matrix Q
Q=gamma*(l%*%t(l))

#Apply the current state of theory, lambda_max=max_i ||q_i||_2
lambda_max=max(sqrt(rowSums(Q^2)))
# Interval for tuning parameter lambda
nl=80
lambda_interval=seq(0,lambda_max,by=lambda_max/(nl-1))
# Vectors that keeps track of the number of non-zero features
features=rep(0,length(lambda_interval))
features_global=rep(0,length(lambda_interval))
fobj=rep(0,length(lambda_interval))

# Number of random starts
r=100

initializev<-function(p){
    x=rnorm(p)
    return(x/sqrt(crossprod(x)))
}

set.seed(5)
vall0=matrix(0,r,p)
for (i in 1:r){
    vall0[i,]=initializev(p)
}


#Solve using Algorithm 1 for the range of lambda_values, keep track of the number of selected features

source("Algorithm1.R")
featurestmp=matrix(0,length(lambda_interval),r+1)
for (i in 1:length(lambda_interval)){
    cat(i)
    lambda=lambda_interval[i]
    fobjtmp=rep(0,r+1)
    for (j in 1:r){
        v0=vall0[j,]
        out=algorithm1_c(Q,lambda,v_init=v0)
        fobjtmp[j]=out$fobj
        featurestmp[i,j]=sum(out$v!=0)
    }
    out1=algorithm1_c(Q,lambda)
    featurestmp[i,r+1]=sum(out1$v!=0)
    fobjtmp[r+1]=out1$fobj
    features_global[i]=min(featurestmp[i,fobjtmp==max(fobjtmp)])
    fobj[i]=max(fobjtmp)
}
#Make a plot of number of selected variables versus tuning parameter 

library(ggplot2)
scenario2=data.frame(lambda=rep(lambda_interval,r+2),features=as.vector(cbind(featurestmp,features_global)),size=c(rep(2,(r+1)*length(lambda_interval)),rep(4,length(lambda_interval))),shape=c(rep(19,(r+1)*length(lambda_interval)),rep(5,length(lambda_interval))),colour=c(rep("red",(r+1)*length(lambda_interval)),rep("black",length(lambda_interval))),alpha=c(rep(0.5,(r+1)*length(lambda_interval)),rep(1,length(lambda_interval))))
pdf(file="../figs/Scenario2globalNew.pdf",width=4,height=4)
ggplot(scenario2,aes(x=lambda,y=features))+geom_point(size=scenario2$size,col=scenario2$colour,shape=scenario2$shape)+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("p=2000 and rank(Q)=1")#+theme_bw()
dev.off()

# rank(Q)=50, C=I, p=500 
######################
p=500
set.seed(95)
n=50
# Create a covariance matrix of rank 5
Xdata=matrix(rnorm(n*p),n,p)
# Scale the data
Xdata=scale(Xdata)
Q=cov(Xdata)


#Apply the current state of theory, lambda_max=max_i ||q_i||_2
lambda_max=max(sqrt(rowSums(Q^2)))
nl=120
lambda_interval=seq(0,lambda_max,by=lambda_max/(nl-1))
# Vectors that keeps track of the number of non-zero features
features=rep(0,length(lambda_interval))
features_global=rep(0,length(lambda_interval))
features_global2=features_global
fobj=rep(0,length(lambda_interval))

# Number of random starts
r=100

initializev<-function(p){
    x=rnorm(p)
    return(x/sqrt(crossprod(x)))
}

set.seed(5)
vall0=matrix(0,r,p)
for (i in 1:r){
    vall0[i,]=initializev(p)
}


#Solve using Algorithm 1 for the range of lambda_values, keep track of the number of selected features

source("Algorithm1.R")
featurestmp=matrix(0,length(lambda_interval),r+1)
for (i in 1:length(lambda_interval)){
    cat(i)
    lambda=lambda_interval[i]
    fobjtmp=rep(0,r+1)
    for (j in 1:r){
        v0=vall0[j,]
        out=algorithm1_c(Q,lambda,v_init=v0)
        fobjtmp[j]=out$fobj
        featurestmp[i,j]=sum(out$v!=0)
    }
    out1=algorithm1_c(Q,lambda)
    featurestmp[i,r+1]=sum(out1$v!=0)
    fobjtmp[r+1]=out1$fobj
    features_global[i]=min(featurestmp[i,fobjtmp==max(fobjtmp)])
    features_global2[i]=max(featurestmp[i,fobjtmp==max(fobjtmp)])
    fobj[i]=max(fobjtmp)
}

#Make a plot of number of selected variables versus tuning parameter
library(ggplot2)
scenario5=data.frame(lambda=rep(lambda_interval,r+2),features=as.vector(cbind(featurestmp,features_global)),size=c(rep(2,(r+1)*length(lambda_interval)),rep(4,length(lambda_interval))),shape=c(rep(19,(r+1)*length(lambda_interval)),rep(5,length(lambda_interval))),colour=c(rep("red",(r+1)*length(lambda_interval)),rep("black",length(lambda_interval))),alpha=c(rep(0.5,(r+1)*length(lambda_interval)),rep(1,length(lambda_interval))))

pdf(file="../figs/Scenario5globalNew.pdf",width=4,height=4)
ggplot(scenario5,aes(x=lambda,y=features))+geom_point(size=scenario5$size,col=scenario5$colour,shape=scenario5$shape)+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("p=500 and rank(Q)=50")
dev.off()

# rank(Q)=50, C=I, p=2000 
######################
p=2000
set.seed(506)
n=50
# Create a covariance matrix of rank 5
Xdata=matrix(rnorm(n*p),n,p)
Xdata=scale(Xdata)
#svdXdata=svd(Xdata)
Q=cov(Xdata)

#Apply the current state of theory, lambda_max=max_i ||q_i||_2
lambda_max=max(sqrt(rowSums(Q^2)))
# Interval for tuning parameter lambda
nl=120
lambda_interval=seq(0,lambda_max,by=lambda_max/(nl-1))
# Vectors that keeps track of the number of non-zero features
features=rep(0,length(lambda_interval))
features_global=rep(0,length(lambda_interval))
fobj=rep(0,length(lambda_interval))

# Number of random starts
# Number of random starts
r=100

initializev<-function(p){
    x=rnorm(p)
    return(x/sqrt(crossprod(x)))
}

set.seed(5)
vall0=matrix(0,r,p)
for (i in 1:r){
    vall0[i,]=initializev(p)
}


#Solve using Algorithm 1 for the range of lambda_values, keep track of the number of selected features

source("Algorithm1.R")
featurestmp=matrix(0,length(lambda_interval),r+1)
for (i in 1:length(lambda_interval)){
    cat(i)
    lambda=lambda_interval[i]
    fobjtmp=rep(0,r+1)
    for (j in 1:r){
        v0=vall0[j,]
        out=algorithm1_c(Q,lambda,v_init=v0)
        fobjtmp[j]=out$fobj
        featurestmp[i,j]=sum(out$v!=0)
    }
    out1=algorithm1_c(Q,lambda)
    featurestmp[i,r+1]=sum(out1$v!=0)
    fobjtmp[r+1]=out1$fobj
    features_global[i]=min(featurestmp[i,fobjtmp==max(fobjtmp)])
    fobj[i]=max(fobjtmp)
}
library(corrplot)
corrplot(featurestmp,is.corr=F)

#Make a plot of number of selected variables versus tuning parameter 
library(ggplot2)
scenario6=data.frame(lambda=rep(lambda_interval,r+2),features=as.vector(cbind(featurestmp,features_global)),size=c(rep(2,(r+1)*length(lambda_interval)),rep(4,length(lambda_interval))),shape=c(rep(19,(r+1)*length(lambda_interval)),rep(5,length(lambda_interval))),colour=c(rep("red",(r+1)*length(lambda_interval)),rep("black",length(lambda_interval))),alpha=c(rep(0.5,(r+1)*length(lambda_interval)),rep(1,length(lambda_interval))))
pdf(file="../figs/Scenario6globalNew.pdf",width=4,height=4)
ggplot(scenario6,aes(x=lambda,y=features))+geom_point(size=scenario6$size,col=scenario6$colour,shape=scenario6$shape)+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("p=2000 and rank(Q)=50")
dev.off()

save(scenario1,scenario2,scenario5,scenario6,file="ScenariosGlobal.rda")
