#include <stdio.h>
#include <stdlib.h>
#include <math.h>

//function for maximum
double max(double a, double b){
    if (a>=b){
        return a;
    }else{
        return b;
    }
}


//function for inner product of two vectors
double mul(double *a, double *b, int* length){
    int index;
    double result=0;
    
    for (index = 0; index < *length; index++){
        result+=a[index] * b[index];
    }
    return result;
}

//calculate l1 norm of vector v
//double l1norm(double *v,int *p){
//    int k;
//    double norm=0;
//    
//    for (k=0;k<*p;k++){
//        norm+=fabs(v[k]);
//    }
//    return(norm);
//}


//Function for one round of coordinate update
void coordUpdate(double *Q, double *lambda,double *v, double *Qv, int *p, double *errf)
{
  int k;
  double fold=0,normv=0,fnew=0;
    
    //calculate new v by soft-thresholding each element of Qv
    for (k=0;k<*p;k++){
        fold+=v[k]*Qv[k]-(*lambda)*fabs(v[k]); // calculate the value of the objective before the update
        v[k]=((Qv[k]>0)-(Qv[k]<0))*max(0,fabs(Qv[k])-(*lambda)/2);
        normv+=v[k]*v[k];
    }
    // if resulting v is non-zero - normalize
    normv=sqrt(normv);
    if (normv!=0){
        for (k=0;k<*p;k++){
            v[k]=v[k]/normv;
        }
    }
    // calculate new value of fobj and Qv
    for (k=0;k<*p;k++){
        Qv[k]=mul(&Q[k*(*p)],v,p);
        fnew+=v[k]*Qv[k]-(*lambda)*fabs(v[k]);
    }
    // calculate difference in objective functions
    *errf=fabs(fnew-fold);
}

//Complete solver for algorithm 1
// Q - symmetric spd matrix, p x p
// lambda - tuning parameter
// v - initial value of vector v, starting point for the algorithm
// fobj - initial value of the objective function
// p - number of variables
// eps - convergence threshold
// max_iter - maximal number of iterations
// iter - current number of iterations
void algorithm1(double *Q, double *lambda,double *v, double *Qv, int *p, double *eps, int *max_iter,int *iter){

  double errf;
  
    *iter=0;
    do{
      ++*iter;
        coordUpdate(Q,lambda,v,Qv,p,&errf);
    }while ((errf>*eps)&&(*iter<*max_iter));
}
